@@ -1,6 +1,6 @@
 """
 /******************************************************************************
- * $Id: core.py 889 2008-09-28 04:17:22Z hobu $
+ * $Id: core.py 1001 2009-02-06 03:03:13Z hobu $
  *
  * Project:  libLAS - http://liblas.org - A BSD library for LAS format data.
  * Purpose:  Python ctypes function calls
@@ -95,6 +95,15 @@ def check_value_free(result, func, cargs):
     free(result)
     return retval
 
+def free_returned_char_p(result, func, cargs):
+
+    size = ctypes.c_int()
+    retvalue = ctypes.string_at(result)
+    free(result)
+    return retvalue
+    
+    
+
 try:
     from numpy import array, ndarray
     HAS_NUMPY = True
@@ -134,23 +143,21 @@ else:
 
 
 def get_version():
-    func = las.LAS_GetVersion
-    size = ctypes.c_int()
-    def errcheck(result, func, argtuple):
-        retval = ctypes.string_at(result, size.value)[:]
-        free(result)
-        return result
-    func.errcheck = errcheck
-    return func()
-
-#version = las.LAS_GetVersion()
+    return las.LAS_GetVersion()
+    
 version = get_version()
 
+las.LAS_GetVersion.restype = ctypes.POINTER(ctypes.c_char)
+las.LAS_GetVersion.errcheck = free_returned_char_p
+
+las.LAS_GetVersion.restype = ctypes.c_char_p
 las.LASError_GetLastErrorNum.restype = ctypes.c_int
 
-las.LASError_GetLastErrorMsg.restype = ctypes.c_char_p
+las.LASError_GetLastErrorMsg.restype = ctypes.POINTER(ctypes.c_char)
+las.LASError_GetLastErrorMsg.errcheck = free_returned_char_p
 
-las.LASError_GetLastErrorMethod.restype = ctypes.c_char_p
+las.LASError_GetLastErrorMethod.restype = ctypes.POINTER(ctypes.c_char)
+las.LASError_GetLastErrorMethod.errcheck = free_returned_char_p
 
 las.LASError_GetErrorCount.restype=ctypes.c_int
 
@@ -521,4 +528,5 @@ las.LASVLR_GetData.restype = ctypes.c_int
 
 las.LASVLR_SetData.argtypes = [ctypes.c_void_p, ctypes.POINTER(ctypes.c_ubyte)]
 las.LASVLR_SetData.errcheck = check_value
-las.LASVLR_SetData.restype = ctypes.c_int
\ No newline at end of file
+las.LASVLR_SetData.restype = ctypes.c_int
+
