@@ -1,7 +1,59 @@
+"""
+/******************************************************************************
+ * $Id: header.py 889 2008-09-28 04:17:22Z hobu $
+ *
+ * Project:  libLAS - http://liblas.org - A BSD library for LAS format data.
+ * Purpose:  Python LASHeader implementation
+ * Author:   Howard Butler, hobu.inc@gmail.com
+ *
+ ******************************************************************************
+ * Copyright (c) 2008, Howard Butler
+ *
+ * All rights reserved.
+ * 
+ * Redistribution and use in source and binary forms, with or without 
+ * modification, are permitted provided that the following 
+ * conditions are met:
+ * 
+ *     * Redistributions of source code must retain the above copyright 
+ *       notice, this list of conditions and the following disclaimer.
+ *     * Redistributions in binary form must reproduce the above copyright 
+ *       notice, this list of conditions and the following disclaimer in 
+ *       the documentation and/or other materials provided 
+ *       with the distribution.
+ *     * Neither the name of the Martin Isenburg or Iowa Department 
+ *       of Natural Resources nor the names of its contributors may be 
+ *       used to endorse or promote products derived from this software 
+ *       without specific prior written permission.
+ * 
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
+ * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
+ * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
+ * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
+ * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
+ * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
+ * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS 
+ * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
+ * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
+ * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
+ * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
+ * OF SUCH DAMAGE.
+ ****************************************************************************/
+ """
 import core
 import datetime
 import guid
+import vlr
 
+def leap_year(year):
+    if (year % 400) == 0:
+        return True
+    elif (year % 100) == 0:
+        return True
+    elif (year % 4) == 0:
+        return False
+    return False
+    
 class Header(object):
     def __init__(self, owned=True, handle=None):
         if handle:
@@ -95,11 +147,12 @@ class Header(object):
         """
         day = core.las.LASHeader_GetCreationDOY(self.handle)
         year = core.las.LASHeader_GetCreationYear(self.handle)
-        
         if year == 0 and day == 0:
             return None
-        
-        return datetime.datetime(year,1,1)+datetime.timedelta(day)
+        if not leap_year(year):
+            return datetime.datetime(year,1,1)+datetime.timedelta(day)
+        else:
+            return datetime.datetime(year,1,1)+datetime.timedelta(day-1)
     def set_date(self, value=datetime.datetime.now()):
         """Set the header's date from a datetime.datetime instance.
 
@@ -115,7 +168,10 @@ class Header(object):
         datetime.datetime(2008, 3, 19, 0, 0)
         """
         delta = value - datetime.datetime(value.year,1,1)
-        core.las.LASHeader_SetCreationDOY(self.handle, delta.days)
+        if not leap_year(value.year):
+            core.las.LASHeader_SetCreationDOY(self.handle, delta.days)
+        else:
+            core.las.LASHeader_SetCreationDOY(self.handle, delta.days + 1)
         core.las.LASHeader_SetCreationYear(self.handle, value.year)
     date = property(get_date, set_date)
     
@@ -303,3 +359,15 @@ class Header(object):
         return core.las.LASHeader_SetMax(self.handle, value[0], value[1], value[2])
     max = property(get_max, set_max)
 
+    def get_proj4(self):
+        return core.las.LASHeader_GetProj4(self.handle)
+    def set_proj4(self, value):
+        return core.las.LASHeader_SetProj4(self.handle, value)
+    proj4 = property(get_proj4, set_proj4)
+    
+    def GetVLR(self, value):
+        return vlr.VLR(handle=core.las.LASHeader_GetVLR(self.handle, value))
+
+    def DeleteVLR(self, value):
+        core.las.LASHeader_DeleteVLR(self.handle, value)
+
